/*****************************************************
Firmware for ATmega8 ROV controller. Outputs for controlling
LEDs and motors. Surface communication via USART module,
hardware interface is RS-485.

Data Types:

int8_t - Signed Char 
uint16_t - Unsigned Int 
uint32_t - Unsigned Long 
int64_t - Signed Long Long

(signed/unsigned) char - 1 byte 
(signed/unsigned) short - 2 bytes 
(signed/unsigned) int - 2 bytes 
(signed/unsigned) long - 4 bytes 
(signed/unsigned) long long - 8 bytes 
float - 4 bytes (floating point) 
double - alias to float


Project : ROV Control Board
AVR Clock frequency: 8,00 MHz

Controls
Thruster Altitude PBx and X
Thruster Right PBX and X
Thruster Right PBX and X

Light PWM on PB1
*****************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>

// USART definitions
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)

// USART Received Data Complete interrupt service routine
ISR(USART_RXC_vect)
{
	uint8_t temp;
	static uint8_t bytesig;
	
// Store value from USART Buffer
	temp = UDR;
	
	if (bytesig == 0xFF){	
	// Set PWM duty
		OCR1A = temp * 4;			// (1023 / 255 = 4)
		bytesig = 0;
	
	}else if (temp ==0xDD){
	// Next byte is output byte
		bytesig = 0xDD;
	
	}else if (temp == 0xFF){
	// Next byte is PWM duty cycle
		bytesig = 0xFF;
	
	}else if (((temp & 128) == 0) & (bytesig == 0xDD)){
	// Must be an output byte
		bytesig = 0;
		PORTB = (temp & 0b00111100);
		PORTD = ((temp & 0b00000011)<<6);
		// Check LED bit
		if (temp & 0b01000000){
			// Start PWM
			// Timer1 Initialization for LED PWM.
			// Fast Mode, Inverted, Prescaler 1:1, 10-bit resolution
			// (TOP value is 0x03FF = 1023) giving 7,8kHz PWM frequency
			// High frequency reduces camera flicker, add a low-pass filter to eliminate
			TCCR1A = (1<<COM1A1)|(0<<COM1A0)|(1<<WGM11)|(1<<WGM10);
			TCCR1B = (0<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10);
		}else{
			// Stop PWM
			TCCR1A = 0;
			TCCR1B = 0;
			PORTB = PORTB & 0b00111100;
		}
	}
}

// Initialize USART Module
void USART_Init(void)
{

	// Enable receiver, with interrupts
	UCSRB = (1<<RXEN)|(0<<TXEN)|(1<<RXCIE);
	
	// Load lower 8-bits of the baud rate value into the low byte of the UBRR register 
	UBRRL = BAUD_PRESCALE;
	// Load upper 8-bits of the baud rate value into the high byte of the UBRR register 
	UBRRH = (BAUD_PRESCALE >> 8);
	
	//Set data frame format: asynchronous mode,no parity, 1 stop bit, 8 bit size
	UCSRC = (1<<URSEL)|(0<<UMSEL)|(0<<UPM1)|(0<<UPM0)|(0<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);

}

// Main Program Loop
int main(void)
{

// PORTB Initialization, Motor outputs and LEDs
PORTB = 0;
DDRB = (1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<PB2)|(1<<PB1);

// PORTC Initialization, set as input
PORTC = 0;
DDRC = 0x00;
// PORTD Initialization
PORTD = 0;
DDRD = (1<<PD7)|(1<<PD6)|(1<<PD2);

// Set to receive mode
PORTD &=~(1<<PD2);

// Initialize USART
USART_Init();

// Enable global interrupts
sei();

// Wait for input
while(1);

return 1;
}